/*
 * Copyright (C) 2010 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3, as published
 * by the Free Software Foundation.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by Ken VanDine <ken.vandine@canonical.com>
 */

public class Client : Gtk.Window
{
  private Gwibber.Service service;
  private Gwibber.Streams streams_service;
  private Gtk.Spinner spinner;
  private Gtk.Alignment spinner_align;
  private GLib.Settings state_settings;
  public Client ()
  {
    Object ();
  }

  construct
  {
    service = new Gwibber.Service ();
    streams_service = new Gwibber.Streams ();
    state_settings = new GLib.Settings ("org.gwibber.state");
    spinner = new Gtk.Spinner ();

    (Gtk.Window)configure_event.connect(on_configure_event);

    /*
    service.loading_started.connect((source) => {
    debug ("Loading started");
    spinner_align.show_all ();
    spinner.show (); 
    spinner.start (); 
    });
    service.loading_complete.connect((source) => {
    debug ("Loading completed");
    spinner_align.hide ();
    spinner.stop (); 
    });
    */
    set_name("Gwibber");
    set_icon_name("gwibber");
    /* FIXME: temp hack, we should install a proper application icon */
    try {
    set_icon_from_file("/usr/share/gwibber/ui/gwibber.svg");
    }catch{}
    set_title("Gwibber");
    destroy.connect(on_window_close);
    set_default_size(400,800);
    /* FIXME: Move to last known location and resize */
    resize(state_settings.get_int("width"), state_settings.get_int("height"));
    move(state_settings.get_int("position-x"), state_settings.get_int("position-y"));

    
    var view = new GwibberGtk.StreamView ();
    var model = streams_service.get_stream_model();
    view.model = model;
    view.sort_order = state_settings.get_string("sort-order");

    view.notify["sort-order"].connect (() => {
      state_settings.set_string("sort-order", view.sort_order);
      });
   

    /**** Create menus ****/
    var menubar = new Gtk.MenuBar();

    // Main Gwibber menu
    var gwibber_menu = new Gtk.Menu();
    var refresh_menu_item = new Gtk.MenuItem.with_mnemonic("_Refresh");
    refresh_menu_item.activate.connect(on_refresh);
    gwibber_menu.append(refresh_menu_item);

    var search_menu_item = new Gtk.MenuItem.with_mnemonic("_Search");
    search_menu_item.activate.connect(on_search);
    gwibber_menu.append(search_menu_item);

    var new_stream_menu_item = new Gtk.MenuItem.with_mnemonic("_New Stream");
    new_stream_menu_item.activate.connect(on_new_stream);
    gwibber_menu.append(new_stream_menu_item);

    var close_stream_menu_item = new Gtk.MenuItem.with_mnemonic("_Close Stream");
    close_stream_menu_item.activate.connect(on_close_stream);
    gwibber_menu.append(close_stream_menu_item);

    var quit_menu_item = new Gtk.MenuItem.with_mnemonic("_Quit");
    quit_menu_item.activate.connect(on_quit);
    gwibber_menu.append(quit_menu_item);

    var gwibber_launcher = new Gtk.MenuItem.with_mnemonic("_Gwibber");
    gwibber_launcher.set_submenu(gwibber_menu);
    menubar.append(gwibber_launcher);
    // End Main Gwibber menu

    // Streams menu
    var streams_menu = new Gtk.Menu ();

    GLib.SList<Gtk.RadioMenuItem> *streams_group = null;

    var stream_messages_menu = new Gtk.RadioMenuItem.with_mnemonic(streams_group, "_Messages");
    streams_group = stream_messages_menu.get_group();
    stream_messages_menu.activate.connect(() => {
      view.stream = "messages";
      });
    if (view.stream == "messages")
      stream_messages_menu.set_active (true);
    streams_menu.append(stream_messages_menu);

    var stream_replies_menu = new Gtk.RadioMenuItem.with_mnemonic(streams_group, "_Replies");
    streams_group = stream_replies_menu.get_group();
    stream_replies_menu.activate.connect(() => {
      view.stream = "replies";
      });
    if (view.stream == "replies")
      stream_replies_menu.set_active (true);
    streams_menu.append(stream_replies_menu);

    var stream_private_menu = new Gtk.RadioMenuItem.with_mnemonic(streams_group, "_Private");
    streams_group = stream_private_menu.get_group();
    stream_private_menu.activate.connect(() => {
      view.stream = "private";
      });
    if (view.stream == "private")
      stream_private_menu.set_active (true);
    streams_menu.append(stream_private_menu);

    var stream_public_menu = new Gtk.RadioMenuItem.with_mnemonic(streams_group, "P_ublic");
    streams_group = stream_public_menu.get_group();
    stream_public_menu.activate.connect(() => {
      view.stream = "public";
      });
    if (view.stream == "public")
      stream_public_menu.set_active (true);
    streams_menu.append(stream_public_menu);

    var stream_links_menu = new Gtk.RadioMenuItem.with_mnemonic(streams_group, "_Links");
    streams_group = stream_links_menu.get_group();
    stream_links_menu.activate.connect(() => {
      view.stream = "links";
    if (view.stream == "links")
      stream_links_menu.set_active (true);
      });
    streams_menu.append(stream_links_menu);

    var stream_images_menu = new Gtk.RadioMenuItem.with_mnemonic(streams_group, "_Images");
    streams_group = stream_images_menu.get_group();
    stream_images_menu.activate.connect(() => {
      view.stream = "images";
      });
    if (view.stream == "images")
      stream_images_menu.set_active (true);
    streams_menu.append(stream_images_menu);

    var stream_videos_menu = new Gtk.RadioMenuItem.with_mnemonic(streams_group, "_Videos");
    streams_group = stream_videos_menu.get_group();
    stream_videos_menu.activate.connect(() => {
      view.stream = "videos";
      });
    if (view.stream == "videos")
      stream_videos_menu.set_active (true);
    streams_menu.append(stream_videos_menu);

    var streams_sort_menu_separator = new Gtk.SeparatorMenuItem ();
    streams_menu.append (streams_sort_menu_separator);

    var streams_sort_menu_item = new Gtk.MenuItem.with_mnemonic ("_Sort");
    var streams_sort_menu = new Gtk.Menu ();
    streams_menu.append (streams_sort_menu_item);
    streams_sort_menu_item.set_submenu (streams_sort_menu);
    
    GLib.SList<Gtk.RadioMenuItem> *streams_sort_group = null;

    var streams_sort_asc_menu = new Gtk.RadioMenuItem.with_mnemonic(streams_sort_group, "_Ascending");
    streams_sort_group = streams_sort_asc_menu.get_group();
    streams_sort_asc_menu.activate.connect(() => {
      view.sort_order = "asc";
      });
    if (view.sort_order == "asc")
      streams_sort_asc_menu.set_active(true);
    streams_sort_menu.append (streams_sort_asc_menu);

    var streams_sort_desc_menu = new Gtk.RadioMenuItem.with_mnemonic(streams_sort_group, "_Descending");
    streams_sort_group = streams_sort_desc_menu.get_group();
    streams_sort_desc_menu.activate.connect(() => {
      view.sort_order = "desc";
      });
    if (view.sort_order == "desc")
      streams_sort_desc_menu.set_active(true);
    streams_sort_menu.append (streams_sort_desc_menu);

    var streams_launcher = new Gtk.MenuItem.with_mnemonic("_Streams");
    streams_launcher.set_submenu(streams_menu);
    menubar.append(streams_launcher);

    // End Streams menu


    // Edit menu
    var edit_menu = new Gtk.Menu();
    var accounts_menu_item = new Gtk.MenuItem.with_mnemonic("_Accounts");
    accounts_menu_item.activate.connect(on_accounts);
    edit_menu.append(accounts_menu_item);

    var preferences_menu_item = new Gtk.MenuItem.with_mnemonic("_Preferences");
    preferences_menu_item.activate.connect(on_preferences);
    edit_menu.append(preferences_menu_item);

    var edit_launcher = new Gtk.MenuItem.with_mnemonic("_Edit");
    edit_launcher.set_submenu(edit_menu);
    menubar.append(edit_launcher);
    // End Edit menu


    // Help menu
    var help_menu = new Gtk.Menu();
    var help_online_menu_item = new Gtk.MenuItem.with_mnemonic("_Get Help Online...");
    help_online_menu_item.activate.connect(load_questions_url);
    help_menu.append(help_online_menu_item);

    var translate_menu_item = new Gtk.MenuItem.with_mnemonic("_Translate This Application...");
    translate_menu_item.activate.connect(load_translate_url);
    help_menu.append(translate_menu_item);

    var bug_menu_item = new Gtk.MenuItem.with_mnemonic("_Report A Problem...");
    bug_menu_item.activate.connect(load_bug_url);
    help_menu.append(bug_menu_item);

    var about_menu_item = new Gtk.MenuItem.with_mnemonic("_About");
    about_menu_item.activate.connect(on_about);
    help_menu.append(about_menu_item);

    var help_launcher = new Gtk.MenuItem.with_mnemonic("_Help");
    help_launcher.set_submenu(help_menu);
    menubar.append(help_launcher);
    // End Help menu
    /**** End menu setup ****/
    

    var main_box = new Gtk.VBox(false, 0);
    main_box.pack_start(menubar, false, true, 0);

    // Tab Bar
    var tabbar = new TabBar ();
    main_box.pack_start (tabbar, false, true, 0);
    tabbar.tab_changed.connect ((item) =>
      {
        view.stream = item.stream;    
      });

    var item = new TabBarItem ("messages", "evolution");
    tabbar.add_item (item);

    item = new TabBarItem ("replies", "mail_reply");
    tabbar.add_item (item);

    item = new TabBarItem ("private", "mail-send-receive");
    tabbar.add_item (item);

    item = new TabBarItem ("links", "web-browser");
    tabbar.add_item (item);

    item = new TabBarItem ("images", "gnome-mime-image");
    tabbar.add_item (item);

    item = new TabBarItem ("videos", "video");
    tabbar.add_item (item);


    // /Tab Bar

    spinner_align = new Gtk.Alignment(0.5f, 0.5f, 0, 0);
    spinner_align.add(spinner);
    spinner.set_size_request (40, 40);
    
    main_box.add (spinner_align);

    main_box.pack_end (view, true, true, 0);
    
    add (main_box);

    show_all ();
    spinner_align.hide ();
  }

  static void on_refresh() {
    debug ("on_refresh");
    var service = new Gwibber.Service();
    service.refresh();
  }

  static void on_preferences() {
    debug ("on_preferences");
    GLib.Pid pid;
    try {
    GLib.Process.spawn_async(null, {"gwibber-preferences"}, null,
      GLib.SpawnFlags.SEARCH_PATH, null, out pid);
    } catch {

    }
  }

  static void on_accounts() {
    debug ("on_accounts");
    GLib.Pid pid;
    try {
    GLib.Process.spawn_async(null, {"gwibber-accounts"}, null,
      GLib.SpawnFlags.SEARCH_PATH, null, out pid);
    } catch {

    }
  }

  
  static void on_close_stream() {
    debug ("on_close_stream");
  }

  static void on_search() {
    debug ("on_search");
  }

  static void on_about() {
    debug ("on_about");
    var about_dialog = new Gtk.AboutDialog ();
    about_dialog.set_program_name("Gwibber");
    about_dialog.set_logo_icon_name("gwibber");
    about_dialog.set_version("2.91.1");
    about_dialog.set_website("http://www.gwibber.com");
    about_dialog.run ();
    about_dialog.destroy ();
  }

  static void on_quit() {
    debug ("on_quit");
    //service.quit();
    Gtk.main_quit();
  }

  static void on_new_stream() {
    debug ("on_new_stream");
  }

  static void on_window_close() {
    debug ("on_window_close");
    Gtk.main_quit();
  }

  static void load_questions_url() {
    debug ("load_questions_url");
  }

  static void load_translate_url() {
    debug ("load_translate_url");
  }

  static void load_bug_url() {
    debug ("load_bug_url");
  }

  bool on_configure_event (Gdk.EventConfigure event) {
    int width, height, x, y;
    get_size(out width, out height);
    get_position (out x, out y);
    state_settings.set_int("width", width);
    state_settings.set_int("height", height);
    state_settings.set_int("position-x", x);
    state_settings.set_int("position-y", x);
    return false;
  }
}

public void main (string[] args)
{
  Gtk.init (ref args);

  var c = new Client ();

  c.show ();
  //w.resize (400, 800);
  //c.position = Gtk.WindowPosition.CENTER;
  Gtk.main ();
}
