/*
 * Copyright (C) 2010 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License version 3, as published 
 * by the Free Software Foundation.

 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranties of 
 * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along 
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by Ken VanDine <ken.vandine@canonical.com>
 */

public class GwibberPreferences : GLib.Object
{
    public Gtk.Builder builder;

    public GwibberPreferences ()
    {
        Object ();
    }

    construct
    {
        try {
            builder = new Gtk.Builder ();
            builder.add_from_file ("data/gwibber-preferences-dialog.ui");
            builder.connect_signals (null);
            var dialog = builder.get_object ("prefs_dialog") as Gtk.Dialog;
            var indicator_server = Indicate.Server.ref_default ();
            if (indicator_server.check_interest(Indicate.Interests.SERVER_DISPLAY)) {
            //if (false) {
                var tray_icon = builder.get_object("tray_icon_frame") as Gtk.Widget;
                tray_icon.hide();
            }
            dialog.show_all ();
        } catch (Error e) {
            error ("Could not load UI: %s", e.message);
        }

        var settings = new GLib.Settings ("org.gwibber.preferences");

        var BOOL_SETTINGS = new GLib.List<string> ();
        var INT_SETTINGS = new GLib.List<string> ();
        var STRING_SETTINGS = new GLib.List<string> ();
        STRING_SETTINGS.append("retweet-style");
        STRING_SETTINGS.append("urlshorter");
        //foreach (string key in STRING_SETTINGS) {
        //    debug (key);
        //    settings.bind(key, builder.get_object(key), "value", GLib.SettingsBindFlags.DEFAULT);
        //}

        INT_SETTINGS.append("interval");
        foreach (string key in INT_SETTINGS) {
            debug (key);
            settings.bind(key, builder.get_object(key), "value", GLib.SettingsBindFlags.DEFAULT);
        }

        BOOL_SETTINGS.append("no-notifications");
        BOOL_SETTINGS.append("notify-mentions-only");
        BOOL_SETTINGS.append("show-notifications");
        BOOL_SETTINGS.append("show-tray-icon");
        BOOL_SETTINGS.append("minimize-to-tray");
        BOOL_SETTINGS.append("hide-taskbar-entry");
        //BOOL_SETTINGS.append("presence-check");
        BOOL_SETTINGS.append("show-fullname");
        BOOL_SETTINGS.append("reply-append-colon");
        BOOL_SETTINGS.append("shorten-urls");
        BOOL_SETTINGS.append("global-retweet");
        BOOL_SETTINGS.append("autostart");
        foreach (string key in BOOL_SETTINGS) {
            debug (key);
            settings.bind(key, builder.get_object(key), "active", GLib.SettingsBindFlags.DEFAULT);
        }
    }

    public void on_close_button_clicked() {
        debug ("on_close_button_clicked");
        Gtk.main_quit();
    }

    public void on_prefs_dialog_destroy_event() {
        debug ("on_prefs_dialog_destroy_event");
        Gtk.main_quit();
    }
}

int main (string[] args) {     
    Gtk.init (ref args);

    var p = new GwibberPreferences ();
    Gtk.main ();

    return 0;
}
