/*
 * Copyright (C) 2010 Neil Jagdish Patel
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * version 3.0 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3.0 for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * Authored by Neil Jagdish Patel <njpatel@gmail.com>
 */

using Gwibber;

namespace GwibberGtk
{
  public class StreamViewTile : Gtk.EventBox
  {
    private const int our_padding = 6;

    private Gtk.Alignment align;
    private Gtk.HBox   main_box;
    private Gtk.Alignment lalignment;
    private Gtk.Alignment ralignment;
    private Gtk.EventBox  icon_box;
    private Gtk.Image  icon;
    private Gtk.VBox   vbox;
    private Gtk.HBox   likes_hbox;
    private Gtk.Label  likes_count;
    private new Gtk.Label  name;
    private Gtk.Label  time;
    private Gtk.Label  message;
    private Gwibber.Utils utils;

    public uint uid = 0;

    private string time_string = "";
    private string reply_url = "";

    public StreamViewTile ()
    {
      Object (above_child:true, visible_window:false);
    }

    construct
    {
      utils = new Gwibber.Utils ();    

      draw.connect (on_draw);

      align = new Gtk.Alignment (0.0f, 0.0f, 1.0f, 1.0f);
      align.set_padding (our_padding, our_padding, our_padding, our_padding);
      add (align);

      main_box = new Gtk.HBox (false, 12);
      align.add (main_box);

      lalignment = new Gtk.Alignment (0.5f, 0.0f, 0.0f, 0.0f);
      ralignment = new Gtk.Alignment (0.5f, 0.0f, 0.0f, 0.0f);
      main_box.pack_start (lalignment, false, false, 0);
      main_box.pack_end (ralignment, false, false, 0);

      icon_box = new Gtk.EventBox ();
      icon = new Gtk.Image.from_icon_name ("stock_person", Gtk.IconSize.DIALOG);
      icon_box.add(icon);
      lalignment.add (icon_box);

      vbox = new Gtk.VBox (false, 0);
      main_box.pack_start (vbox, true, true, 0);
      
      var hbox = new Gtk.HBox (false, 0);
      vbox.pack_start (hbox, false, false, 2);
      
      name = new Gtk.Label ("");
      name.set_markup ("<b></b>");
      name.set_alignment (0.0f, 0.5f);
      hbox.pack_start (name, true, true, 0);

      time = new Gtk.Label ("");
      time.set_markup ("");
      time.set_alignment (0.0f, 1.0f);
      hbox.pack_end (time, false, false, 0);

      message = new Gtk.Label ("");
      //message.selectable = true;
      message.set_single_line_mode (false);
      message.set_line_wrap (true);
      message.set_line_wrap_mode (Pango.WrapMode.WORD_CHAR);
      message.set_alignment (0.0f, 0.0f);
      vbox.pack_start (message, false, false, 2);

      likes_hbox = new Gtk.HBox (false, 0);
      // FIXME: Find a theme icon
      var likes_icon = new Gtk.Image.from_file ("/usr/share/gwibber/ui/icons/streams/16x16/favorites.png");
      likes_hbox.pack_start (likes_icon, false, false, 2);
      likes_count = new Gtk.Label ("");
      likes_hbox.pack_start (likes_count, false, false, 0);
      vbox.pack_start (likes_hbox, false, false, 2);

      enter_notify_event.connect (() => {
        update_time_area ();
        return false;
      });
      icon_box.enter_notify_event.connect (() => {
        update_time_area ();
        return false;
      });
      message.enter_notify_event.connect (() => {
        update_time_area ();
        return false;
      });
      leave_notify_event.connect (() => {
        update_time_area ();
        return false;
      });
    }

    private bool on_draw (Cairo.Context cr)
    {
      Gtk.Allocation alloc;
      get_allocation (out alloc);

      cr.set_line_width (1.0);

      cr.set_source_rgba (1.0, 1.0, 1.0, 1.0);
      cr.move_to (alloc.x, 0.5);
      cr.line_to (alloc.x + alloc.width, 0.5);
      cr.stroke ();

      var pat = new Cairo.Pattern.linear (0, 0, 0, alloc.height);
      pat.add_color_stop_rgba (0, 1.0, 1.0, 1.0, 0.8);
      pat.add_color_stop_rgba (1.0, 1.0, 1.0, 1.0, 0.5);

      cr.rectangle (0, 0, alloc.width, alloc.height);
      cr.set_source (pat);
      cr.fill ();

      cr.set_source_rgba (0.0, 0.0, 0.0, 0.1);
      cr.move_to (alloc.x, alloc.height - 0.5);
      cr.line_to (alloc.x + alloc.width, alloc.height - 0.5);
      cr.stroke ();

      propagate_draw (align, cr);

      return true;
    }

    public void reset ()
    {
      hide ();
    }

    private void update_time_area ()
    {
      var str = "<span font_weight='light' font_size='small' color='#999'>";

      int x, y;
      get_pointer (out x, out y);
      Gtk.Allocation a;
      get_allocation (out a);
      if (x > 0 && x < a.width && y > 0 && y < a.height)
      {
        str += "<a href='conversation'>conversation</a> ";
        str += "<a href='retweet'>retweet</a> ";
        str += "<a href='reply'>reply</a>";
      }
      else
        str += time_string;

      str += "</span>";

      time.set_markup (str);
    }

    public void set_details (string _account,
                             string _stream,
                             string _service,
                             string _sender,
                             bool _from_me,
                             string _timestamp,
                             string _message,
                             string _html,
                             string _icon_uri,
                             string _url,
                             string _source,
                             string _timestring,
                             string _reply_nick,
                             string _reply_url,
                             double _likes)
    {
      name.set_markup ("<b>" + _sender + "</b>");

      if (_from_me)
        icon_box.reparent(ralignment);
      else
        icon_box.reparent(lalignment);

      icon.set_from_file(_icon_uri);
      //icon_box.button_press_event.disconnect();
      icon_box.button_press_event.connect((event) => {
        /* FIXME: Display a user view with user info and posts instead of 
        launching external browser */
        try {
        AppInfo.launch_default_for_uri (_url, null);
        } catch {
          warning ("Unable to launch: " + _url);
        }
        return true;
        });

      if (_likes == 0) {
        likes_hbox.hide();
      } else if (_likes < 0) {
        likes_count.set_markup("");
        likes_hbox.show();
      } else if (_likes == 1) {
        likes_count.set_markup("<span font_size='small'>" + _likes.to_string() + " " + _("person liked this") + "</span>");
        likes_hbox.show();
      } else if (_likes > 1) {
        likes_count.set_markup("<span font_size='small'>" + _likes.to_string() + " " + _("people liked this") + "</span>");
        likes_hbox.show();
      }

      message.set_markup (_html);

      if (_reply_nick != null && _reply_nick.length > 0)
        reply_url = _reply_url;
      else
        reply_url = "";

      time_string = utils.generate_time_string ((uint)long.parse(_timestamp));
      update_time_area ();

      queue_resize ();
      show ();
    }                        
  }
}
